@@ -1049,10 +1049,11 @@ class Client(object):
         ----------
         model_id : int
             Integer id of the model to be applied
-        samples : dict 
-            Dict of model inputs- keys are sample ids, 
+        samples : dict or pandas.DataFrame
+            Model inputs- if it is a dict, the keys are sample ids, 
             and values are dicts of model inputs. 
-            The returned dict will have the same keys.
+            If it is a DataFrame, the index contains sample ids,
+            and the columns are the model inputs.
         run_featurization : bool
             Whether or not to run the model's featurization on the samples
         run_transformation : bool
@@ -1063,6 +1064,8 @@ class Client(object):
         resp : dict
             Dict containing model results or error reports
         """
+        if isinstance(samples,pd.DataFrame):
+            samples = samples.to_dict(orient='index')
         pid_resp = self.session.get(
             self.service_url+'model_process_id', 
             data={
@@ -1125,7 +1128,7 @@ class Client(object):
                     'file_ext':'.csv'
                 }
             )
-            df = pd.read_csv(BytesIO(res._content),index_col='name')
+            df = pd.read_csv(BytesIO(res._content),index_col='id')
             if output_path:
                 df.to_csv(output_path)
             return {'success':True,'data':res._content,'df':df}
