@@ -52,7 +52,7 @@ class SubstancesClient(object):
 
     # SUBSTANCES
 
-    def add_substance(self,name,descriptors={},properties={},documents={},source='',other_names=[]):
+    def add_substance(self,name,source='',other_names=[],descriptors={},properties={},documents={}):
         """Add a Substance to the database.
 
         A Substance is defined by a name and a set of descriptors.
@@ -65,6 +65,16 @@ class SubstancesClient(object):
             A name for the Substance.
             If this name already exists in the database,
             the add_substance operation will fail.
+        source : str
+            A representation of the structure of the Substance,
+            sufficient for extracting descriptors.
+            For example, a molecule might be represented by a SMILES string,
+            and a crystalline solid might be represented by a CIF file.
+        other_names : list
+            Any number of additional names to assign to the Substance.
+            Substance names must be unique, so if any of these names
+            are already assigned to substances in the database,
+            this operation will fail.
         descriptors : dict
             Keys are descriptor names, and values are descriptor values.
             Descriptors are used to encode substances for modeling,
@@ -81,16 +91,6 @@ class SubstancesClient(object):
             Keys are document names, and values are document full texts.
             This should be used to store any substance data
             that do not fit neatly as descriptors or properties.
-        source : str
-            A representation of the structure of the Substance,
-            sufficient for extracting descriptors.
-            For example, a molecule might be represented by a SMILES string,
-            and a crystalline solid might be represented by a CIF file.
-        other_names : list
-            Any number of additional names to assign to the Substance.
-            Substance names must be unique, so if any of these names
-            are already assigned to substances in the database,
-            this operation will fail.
 
         Returns
         -------
@@ -115,6 +115,51 @@ class SubstancesClient(object):
         else:
             return {'success':False,'status_code':resp.status_code}
 
+    def update_substance(self,substance_name,source='',other_names=[],descriptors={},properties={},documents={}):
+        """Update the attributes of an existing substance
+
+        Parameters
+        ----------
+        substance_name : str
+            A name that is already assigned to the Substance of interest. 
+        source : str
+            A string representing of the structure of the Substance,
+            sufficient for extracting descriptors.
+        other_names : list
+            Any number of additional names to assign to the Substance.
+        descriptors : dict
+            Keys are descriptor names, and values are descriptor values.
+        properties : dict
+            Keys are property names, and values are property values.
+        documents : dict
+            Keys are document names, and values are document full texts.
+
+        Returns
+        -------
+        response : dict
+            Dict containing status report and results.
+        """
+        resp = self.session.post(
+            self.service_url+'update_substance', 
+            headers={'x-api-key':self.api_key},
+            json={
+                'substance_name':substance_name,
+                'source':source,
+                'other_names':other_names,
+                'descriptors':descriptors,
+                'properties':properties,
+                'documents':documents
+            }
+        )
+        if resp.status_code == 200:
+            respdata = resp.json()
+            return respdata
+        else:
+            return {'success':False,'status_code':resp.status_code}
+
+
+    # DESCRIPTORS
+
     def check_substance_names(self,substance_names):
         """Check for existence of substances by name.
 
@@ -171,7 +216,6 @@ class SubstancesClient(object):
         else:
             return {'success':False,'status_code':resp.status_code}
 
-    # DESCRIPTORS
 
     def get_descriptors(self,substance_names,descriptor_names=[]):
         """Get substance descriptors by providing substance names.
@@ -203,18 +247,17 @@ class SubstancesClient(object):
             self.service_url+'get_descriptors', 
             headers={'x-api-key':self.api_key},
             json={
-                'substance_names':substance_names,
-                'descriptor_names':descriptor_names
+                'substance_names':substance_names
             }
         )
         if resp.status_code == 200:
             respdata = resp.json()
-            all_cols = set()
-            for subsnm,subsdescs in respdata['descriptors'].items():
-                all_cols = all_cols.union(set(subsdescs.keys()))
-            ddf = pd.DataFrame(index=respdata['descriptors'].keys(),columns=all_cols)
-            for subsnm,subsdescs in respdata['descriptors'].items():
-                ddf.loc[subsnm,subsdescs.keys()] = subsdescs.values()
+            ddf = pd.DataFrame.from_dict(respdata['descriptors'],orient='index')
+            if descriptor_names:
+                descnms = [descnm for descnm in descriptor_names if descnm in ddf.columns]
+                ddf = ddf[descnms].copy()
+                for subsnm in respdata['descriptors'].keys():
+                    respdata['descriptors'][subsnm] = ddf.loc[subsnm].to_dict()
             respdata['descriptor_table'] = ddf
             return respdata
         else:
